## Copyright (C) 2009-2013 M. Oliveira and F. Nogueira
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.
##
## $Id: libxc.m4 800 2013-10-09 11:18:39Z micael $

AC_DEFUN([ACX_LIBXC], [

dnl Check if the library was given in the command line
dnl if not, use environment variables or defaults
AC_ARG_WITH(libxc_libs, [AS_HELP_STRING([--with-libxc-libs=<libs>], 
            [Use libxc libraries <libs>],[32])])
AC_ARG_WITH(libxc_path, [AS_HELP_STRING([--with-libxc-path=<path>], 
            [Path to libxc install directory],[32])])
AC_ARG_WITH(libxc_libdir, [AS_HELP_STRING([--with-libxc-libdir=<path>], 
            [Path to the libxc lib directory],[32])])
AC_ARG_WITH(libxc_includedir, [AS_HELP_STRING([--with-libxc-includedir=<path>], 
            [Path to the libxc include directory],[32])])


acx_libxc_ok="no"
internal_libxc="no"
compile_libxc="no"

dnl Backup LIBS and FCFLAGS
acx_libxc_save_LIBS="$LIBS"
acx_libxc_save_FCFLAGS="$FCFLAGS"

dnl The tests
AC_MSG_CHECKING([for libxc])

dnl The following program should work with all version of libxc
testprog="AC_LANG_PROGRAM([],[
    use xc_f03_lib_m
    implicit none

    integer :: i
    i = XC_EXCHANGE
])"


# set from environment variable, if not blank
if test ! -z "$LIBXC_LIBS"; then
  LIBS="$LIBXC_LIBS"
  FCFLAGS="$LIBXC_INCS $acx_libxc_save_FCFLAGS"
dnl $acx_libxc_save_LIBS"
  AC_LINK_IFELSE($testprog, [acx_libxc_ok=yes], [])
fi

# set from configure flags
if test x"$acx_libxc_ok" = xno ; then
  #
  LIBXC_LIBS="-lxcf90 -lxcf03 -lxc"
  LIBXC_INCS=""
  #
  if test -d "$with_libxc_path"; then
     libxc_incdir="$with_libxc_path/include"
     libxc_libdir="$with_libxc_path/lib"
  fi
  if test -d "$with_libxc_includedir"; then libxc_incdir="$with_libxc_includedir" ; fi
  if test -d "$with_libxc_libdir";     then libxc_libdir="$with_libxc_libdir" ; fi
  #
  # dynamic linkage, separate Fortran interface
  if test ! -z "$libxc_libdir"; then LIBXC_LIBS="-L$libxc_libdir -lxcf90 -lxcf03 -lxc"; fi
  if test ! -z "$libxc_incdir"; then LIBXC_INCS="$IFLAG$libxc_incdir"; fi
  #
  if test ! -z "$with_libxc_libs" ; then LIBXC_LIBS="$with_libxc_libs" ; fi
  if test ! -z "$with_libxc_incs" ; then LIBXC_INCS="$with_libxc_incs" ; fi

  LIBS="$LIBXC_LIBS"
  FCFLAGS="$LIBXC_INCS $acx_libxc_save_FCFLAGS"

dnl $acx_libxc_save_LIBS"
  AC_LINK_IFELSE($testprog, [acx_libxc_ok=yes], [])
  
  # if dynamic / user provided linkage did not work, try static one
  if test x"$acx_libxc_ok" = xno && test ! -z "$libxc_libdir"; then
    LIBXC_LIBS="$libxc_libdir/libxcf90.a $libxc_libdir/libxcf03.a  $libxc_libdir/libxc.a"
    LIBS="$LIBXC_LIBS"
dnl $acx_libxc_save_LIBS"
    AC_LINK_IFELSE($testprog, [acx_libxc_ok=yes], [])
  fi
fi


dnl The following programs will only work with specific version of libxc
testprog_5x="AC_LANG_PROGRAM([],[
  use xc_f03_lib_m
  implicit none
  type(xc_f03_func_t) :: p
  real(selected_real_kind(14,200)) :: ext_params
  call xc_f03_func_set_ext_params(p,(/ext_params/))
])"

dnl The following programs will only work with specific version of libxc
testprog_4x="AC_LANG_PROGRAM([],[
  use xc_f90_lib_m
  use xc_f90_types_m
  implicit none
  type(xc_f90_pointer_t) :: p
  real(xc_f90_kind) :: ext_params
  call xc_f90_func_set_ext_params(p,ext_params)
])"

testprog_21="AC_LANG_PROGRAM([],[
  use xc_f90_lib_m
  implicit none

  integer :: i
  i = XC_GGA_X_LV_RPW86])"

testprog_203="AC_LANG_PROGRAM([],[
  use xc_f90_lib_m
  use xc_f90_types_m
  implicit none
  type(xc_f90_pointer_t) :: conf
  real(8) :: alpha
  real(8) :: gamma
  call xc_f90_hyb_gga_xc_hse_set_par(conf,alpha,gamma)])"

testprog_20="AC_LANG_PROGRAM([],[
  use xc_f90_lib_m
  implicit none
  integer :: major
  integer :: minor
  call xc_f90_version(major, minor)])"

testprog_12="AC_LANG_PROGRAM([],[
  use xc_f90_lib_m
  implicit none

  integer :: i
  i = XC_GGA_X_AIRY])"

testprog_11="AC_LANG_PROGRAM([],[
  use xc_f90_lib_m
  implicit none

  integer :: i
  i = XC_KINETIC])"

dnl Get libxc version
if test x"$acx_libxc_ok" = xyes; then
AC_LINK_IFELSE($testprog_11,  [acx_libxc_version=110, acx_libxc_ok=no ], [])
AC_LINK_IFELSE($testprog_12,  [acx_libxc_version=120, acx_libxc_ok=no ], [])
AC_LINK_IFELSE($testprog_20,  [acx_libxc_version=200, acx_libxc_ok=no ], [])
AC_LINK_IFELSE($testprog_203, [acx_libxc_version=203, acx_libxc_ok=no ], [])
AC_LINK_IFELSE($testprog_21,  [acx_libxc_version=210, acx_libxc_ok=no ], [])
AC_LINK_IFELSE($testprog_4x,  [acx_libxc_version=400, acx_libxc_ok=no ], [])
AC_LINK_IFELSE($testprog_5x,  [acx_libxc_version=5,   acx_libxc_ok=yes ], [])
AC_DEFINE_UNQUOTED([LIBXC_VERSION],[$acx_libxc_version],[Defined the LIBXC version.])
AC_MSG_RESULT([Found external LibXC version=$acx_libxc_version (should be 5)])
fi

dnl Finally, execute ACTION-IF-FOUND/ACTION-IF-NOT-FOUND:
if test x"$acx_libxc_ok" = xyes; then
  compile_libxc=no
  internal_libxc=no
  #
  AC_DEFINE(HAVE_LIBXC, 1, [Defined if you have the LIBXC library.])
fi

if test x"$acx_libxc_ok" = xno; then
  have_configured="no"
  internal_libxc="yes"
  # version 5 is used
  #LIBXC_LIBS="${extlibs_path}/${FCKIND}/${FC}/lib/libxcf90.a ${extlibs_path}/${FCKIND}/${FC}/lib/libxcf03.a  ${extlibs_path}/${FCKIND}/${FC}/lib/libxc.a"
  LIBXC_LIBS="-L${extlibs_path}/${FCKIND}/${FC}/lib/ -lxcf90 -lxcf03 -lxc"
  LIBXC_INCS="$IFLAG${extlibs_path}/${FCKIND}/${FC}/include"
  if test -e "${extlibs_path}/${FCKIND}/${FC}/lib/libxc.a" && test -e "${extlibs_path}/${FCKIND}/${FC}/lib/libxcf90.a" && test -e ${extlibs_path}/${FCKIND}/${FC}/lib/libxcf03.a; then
    compile_libxc="no"
    AC_MSG_RESULT([Compatible external LibXC not found/specified. Found internal already compiled.])
  else
    compile_libxc="yes"
    AC_MSG_RESULT([Compatible external LibXC not found/specified. Internal to be compiled.])
  fi
fi 

AC_SUBST(LIBXC_LIBS)
AC_SUBST(LIBXC_INCS)
AC_SUBST(compile_libxc)
AC_SUBST(internal_libxc)
FCFLAGS="$acx_libxc_save_FCFLAGS"
LIBS="$acx_libxc_save_LIBS"
])dnl ACX_LIBXC

